/*
 * File......: ACCTADJ.PRG
 * Author....: Jo W. French dba Practical Computing
 * CIS_ID....: 74731,1751
 * Date......: $Date:   02 Apr 1991 18:21:50  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/acctadj.prv  $
 * 
 * The functions contained herein are the original work of Jo W. French
 * and are placed in the public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/acctadj.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:21:50   GLENN
*  Nanforum Toolkit
 *
 */

/*  $DOC$
 *  $FUNCNAME$
 *     FT_ACCTADJ()
 *  $CATEGORY$
 *     Date/Time
 *  $ONELINER$
 *     Adjust beginning or ending fiscal pd. dates to acctg. dates
 *  $SYNTAX$
 *     FT_ACCTADJ( [<dGivenDate>], [<lIsEnd>] ) -> dDate
 *  $ARGUMENTS$
 *     <dGivenDate> is any valid date in any valid format which is the
 *     beginning or end of a fiscal period.  Defaults to DATE() if
 *     not supplied.
 *
 *     <lIsEnd> is a logical variable. .F. = adjust for beginning of
 *     period, .T. = adjust for end of period.  Defaults to beginning
 *     of period.
 *  $RETURNS$
 *     An adjusted date.
 *  $DESCRIPTION$
 *     If given start is in last 3 days (nTemp > 3) of work week, this week's
 *     data was in prior period. If given end is in last 4 days (nTemp > 2)
 *     of work week then this week's data is in current period.
 *  $EXAMPLES$
 *     dDate := Ctod( "01/31/91" )
 *     ? FT_ACCTADJ( dDate )        // 02/03/91   (beginning of fiscal pd.)
 *     ? FT_ACCTADJ( dDate, .T. )   // 02/02/91   (end of fiscal pd.)
 *
 *     dDate := Ctod( "02/28/91" )
 *     ? FT_ACCTADJ( dDate )        // 03/03/91   (beginning of fiscal pd.)
 *     ? FT_ACCTADJ( dDate, .T. )   // 03/02/91   (end of fiscal pd.)
 *
 *     dDate := Ctod( "03/31/91" )
 *     ? FT_ACCTADJ( dDate )        // 03/31/91   (beginning of fiscal pd.)
 *     ? FT_ACCTADJ( dDate, .T. )   // 03/30/91   (end of fiscal pd.)
 *  $SEEALSO$
 *     FT_DATECNFG()
 *  $END$
*/

FUNCTION FT_ACCTADJ(dGivenDate, lIsEnd)

LOCAL nTemp

dGivenDate := IIF(VALTYPE(dGivenDate) != 'D', DATE(), dGivenDate)
lIsEnd := IIF(VALTYPE(lIsEnd) != 'L', .F., lIsEnd)
nTemp := FT_DAYTOBOW(dGivenDate)

IF nTemp > ( 2 + IF(!lIsEnd, 1, 0) )
  dGivenDate += ( 7 - nTemp )      // Next Week Start (This Week End + 1)
ELSE
  dGivenDate -= nTemp              // This Week Start (Prior Week End + 1)
ENDIF

dGivenDate := dGivenDate - IIF(lIsEnd, 1 , 0)

RETURN dGivenDate

