/*
 * File......: BITSET.PRG
 * Author....: Forest Belt, Computer Diagnostic Services, Inc.
 * Date......: $Date:   02 Apr 1991 18:22:12  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/bitset.prv  $
 * 
 * This is an original work by Forest Belt and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/bitset.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:22:12   GLENN
*  Nanforum Toolkit
 *
 */



/*  $DOC$
 *  $FUNCNAME$
 *     FT_BITSET()
 *  $CATEGORY$
 *     String
 *  $ONELINER$
 *     Set selected bit in a byte
 *  $SYNTAX$
 *     FT_BITSET(cByte, nBitpos) -> cByte
 *  $ARGUMENTS$
 *     <cByte> is a character from CHR(0) to CHR(255).
 *
 *     <nBitPos> is a number from 0 to 7 conforming to standard right-to-left
 *     bit numbering convention and representing the position of the bit
 *     within the byte.
 *  $RETURNS$
 *     Returns new byte, with designated bit set.  If parameters are faulty,
 *     returns NIL.
 *  $DESCRIPTION$
 *     In effect, ORs argument byte with a byte that has only the target bit
 *     set.  If bit is already set, it remains set.
 *     Note: Calls FT_ISBIT() in this module.
 *  $EXAMPLES$
 *     This code would set bit 4 in a byte represented by chr(107):
 *
 *          cNewbyte := FT_BITSET( CHR(107), 4 )
 *          ? ASC( cNewbyte )             // result: 123
 *          ? cNewbyte                    // result: '{'
 *
 *
 *     This code would set bit 5 in the byte represented by th letter 'A'.
 *
 *          ? FT_BITSET( 'A', 5 )         // result: 'A'
 *  $END$
 */

FUNCTION FT_BITSET(cInByte, nBitpos)

  LOCAL xByte

  IF valtype(cInbyte) != "C" .or. valtype(nBitpos) != "N"  // parameter check
     xByte := NIL
  ELSE
     IF (nBitPos > 7) .or. (nBitPos < 0) .or. (nBitPos != int(nBitPos))
        xByte := NIL
     ELSE
        xByte := iif( FT_ISBIT(cInByte, nBitpos), cInByte, ;
                       chr(asc(cInByte) + (2 ^ nBitpos)))
     ENDIF
  ENDIF

RETURN xByte

