/*
 * File......: BYT2BIT.PRG
 * Author....: Forest Belt, Computer Diagnostic Services, Inc.
 * Date......: $Date:   02 Apr 1991 18:22:16  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/byt2bit.prv  $
 * 
 * This is an original work by Forest Belt and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/byt2bit.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:22:16   GLENN
*  Nanforum Toolkit
 *
 */


/*  $DOC$
 *  $FUNCNAME$
 *     FT_BYT2BIT()
 *  $CATEGORY$
 *     Conversion
 *  $ONELINER$
 *     Convert byte to string of 1's and 0's
 *  $SYNTAX$
 *     FT_BYT2BIT( cByte ) -> cBitPattern
 *  $ARGUMENTS$
 *     <cByte> is the byte to convert.
 *  $RETURNS$
 *     9-character string, consisting of 1's and 0's, representing bits 0
 *     through 7 of parameter byte, with space between bits 3 and 4.  Returns
 *     NIL if parameters are faulty.
 *  $DESCRIPTION$
 *     Can be used to show results of bit manipulation, both before and after.
 *     Binary representation follows right-to-left convention of bit position
 *     numbering, 0 through 7.  Space between high and low nibbles for clarity
 *     and easy comparison to hexadecimal notation.
 *  $EXAMPLES$
 *     These three code lines perform a bitwise AND on bytes with values of
 *     chr(20) and chr(36), and deliver the result as a string in binary (bit)
 *     format.
 *
 *          ? FT_BYT2BIT(chr(20))         // byte1: '0001 0100'
 *          ? FT_BYT2BIT(chr(36))         // byte2: '0010 0100'
 *
 *          ? FT_BYT2BIT(FT_BYTEAND(chr(20), chr(36)))
 *                             // result: '0000 0100'
 *  $END$
 */

FUNCTION FT_BYT2BIT(cByte)

  local nCounter, xBitstring

  IF valtype(cByte) != "C"
     xBitString := NIL
  ELSE
     xBitString := ""
     FOR nCounter := 7 TO 0 step -1
        xBitString += iif(FT_ISBIT(cByte, nCounter), "1", "0")
     NEXT
  ENDIF

RETURN xBitString

