/*
 * File......: BYT2HEX.PRG
 * Author....: Forest Belt, Computer Diagnostic Services, Inc.
 * Date......: $Date:   02 Apr 1991 18:22:18  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/byt2hex.prv  $
 * 
 * This is an original work by Forest Belt and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/byt2hex.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:22:18   GLENN
*  Nanforum Toolkit
 *
 */


/*  $DOC$
 *  $FUNCNAME$
 *     FT_BYT2HEX()
 *  $CATEGORY$
 *     Conversion
 *  $ONELINER$
 *     Convert byte to hexadecimal version of its binary value
 *  $SYNTAX$
 *     FT_BYT2HEX( cByte ) -> cHexValue
 *  $ARGUMENTS$
 *     <cByte> is the byte to convert.
 *  $RETURNS$
 *     Three-character string, consisting of two digits of hexadecimal
 *     notation and letter 'h' to signify hex.  Returns NIL if parameters are
 *     faulty.
 *  $DESCRIPTION$
 *     Can be used to show results of bit manipulation, both before and after.
 *  $EXAMPLES$
 *     These three code lines perform a bitwise AND on bytes with values of
 *     CHR(20) and CHR(36), and deliver the result as a string in binary (bit)
 *     format.
 *
 *          ? FT_BYT2HEX(CHR(20))         // byte1: '14h'
 *          ? FT_BYT2HEX(CHR(36))         // byte2: '24h'
 *
 *          ? FT_BYT2HEX(FT_BYTEAND(chr(20), chr(36)))
 *                             // result: '04h'
 *  $END$
 */

FUNCTION FT_BYT2HEX(cByte)

  local cHexTable := "0123456789ABCDEF"
  local xHexString

  if valtype(cByte) != "C"
     xHexString := NIL
  else
     xHexString := substr(cHexTable, int(asc(cByte) / 16) + 1, 1) ;
                 + substr(cHexTable, int(asc(cByte) % 16) + 1, 1) ;
                 + "h"
  endif

RETURN xHexString

