/*
 * File......: BYTEAND.PRG
 * Author....: Forest Belt, Computer Diagnostic Services, Inc.
 * Date......: $Date:   02 Apr 1991 18:22:20  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/byteand.prv  $
 * 
 * This is an original work by Forest Belt and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/byteand.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:22:20   GLENN
*  Nanforum Toolkit
 *
 */



/*  $DOC$
 *  $FUNCNAME$
 *     FT_BYTEAND()
 *  $CATEGORY$
 *     String
 *  $ONELINER$
 *     AND two ASCII characters
 *  $SYNTAX$
 *     FT_BYTEAND(cByte1, cByte2) -> cByte
 *  $ARGUMENTS$
 *     <cByte1> and <cByte2> are characters from CHR(0) TO CHR(255).
 *     May be passed in CHR() form, as character literals, or as expressions
 *     evaluating to CHR() values.
 *  $RETURNS$
 *     Returns resulting byte, in CHR() form.  If parameters are faulty,
 *     returns NIL.
 *  $DESCRIPTION$
 *     Can be used for any masking operation.  In effect, this is a bit-by-bit
 *     (bitwise) AND operation.  Equivalent to AND assembler instruction.
 *  $EXAMPLES$
 *     This code would mask out the high nibble (four most significant bits)
 *     of the byte represented by chr(123) and leave the low nibble bits as in
 *     the parameter byte.
 *
 *          cNewbyte := FT_BYTEAND(chr(123), chr(15))
 *          ? asc(cNewbyte)  // result: 11
 *          ? cNewbyte       // result: non-printing character
 *  $END$
 */

FUNCTION FT_BYTEAND(cByte1, cByte2)

  LOCAL nCounter, xByte

  IF valtype(cByte1) != "C" .or. valtype(cByte2) != "C" // parameter check
     xByte := NIL
  ELSE
     xByte := chr(0)
     for nCounter := 0 to 7           // test each bit position
        if FT_ISBIT(cByte1, nCounter) .and. FT_ISBIT(cByte2, nCounter)
           xByte := FT_BITSET(xByte, nCounter)
        endif
     next
  ENDIF

RETURN xByte

