/*
 * File......: BYTENOT.PRG
 * Author....: Forest Belt, Computer Diagnostic Services, Inc.
 * Date......: $Date:   02 Apr 1991 18:22:22  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/bytenot.prv  $
 * 
 * This is an original work by Forest Belt and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/bytenot.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:22:22   GLENN
*  Nanforum Toolkit
 *
 */



/*  $DOC$
 *  $FUNCNAME$
 *     FT_BYTENOT()
 *  $CATEGORY$
 *     String
 *  $ONELINER$
 *     NOT two ASCII characters
 *  $SYNTAX$
 *     FT_BYTENOT(cByte) -> cByte
 *  $ARGUMENTS$
 *     cByte is a character from CHR(0) to CHR(255).
 *     May be passed in CHR() form, as character literal, or
 *     as expression evaluating to CHR() value.
 *  $RETURNS$
 *     Returns resulting byte, in CHR() form.  If parameters are faulty,
 *     returns NIL.
 *  $DESCRIPTION$
 *     Can be used for bitwise byte manipulation.  In effect, this is a
 *     bit-by-bit (bitwise) NOT operation.  Equivalent to NOT assembler
 *     instruction.
 *  $EXAMPLES$
 *     This code performs a bitwise NOT on byte represented by CHR(32):
 *
 *          cNewbyte := FT_BYTENOT( CHR(32) )
 *          ? ASC( cNewbyte )     // result: 223
 *  $END$
 */


FUNCTION FT_BYTENOT(cByte)

  LOCAL nCounter, xByte

  IF valtype(cByte) != "C"
     xByte := NIL
  ELSE
     xByte := chr(0)
     FOR nCounter := 0 to 7           // test each bit position
        IF .not. FT_ISBIT(cByte, nCounter)
           xByte := FT_BITSET(xByte, nCounter)
        ENDIF
     NEXT
  ENDIF

RETURN xByte

