/*
 * File......: BYTEXOR.PRG
 * Author....: Forest Belt, Computer Diagnostic Services, Inc.
 * Date......: $Date:   02 Apr 1991 18:22:26  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/bytexor.prv  $
 * 
 * This is an original work by Forest Belt and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/bytexor.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:22:26   GLENN
*  Nanforum Toolkit
 *
 */



/*  $DOC$
 *  $FUNCNAME$
 *     FT_BYTEXOR()
 *  $CATEGORY$
 *     String
 *  $ONELINER$
 *     XOR two ASCII characters
 *  $SYNTAX$
 *     FT_BYTEXOR(cByte1, cByte2) -> cByte
 *  $ARGUMENTS$
 *     <cByte1> and <cByte2> are characters from CHR(0) to CHR(255).
 *     May be passed in CHR() form, as character literals, or
 *     as expressions evaluating to CHR() values.
 *  $RETURNS$
 *     Returns resulting byte, in CHR() form.  If parameters are faulty,
 *     returns NIL.
 *  $DESCRIPTION$
 *     Can be used for bitwise byte manipulation.  In effect, this is a
 *     bit-by-bit (bitwise) XOR operation.  Equivalent to XOR assembler
 *     instruction.
 *  $EXAMPLES$
 *     This code performs a bitwise XOR on two bytes represented
 *     by chr(32) and chr(55):
 *
 *          cNewbyte := FT_BYTEXOR( CHR(32), CHR(55) )
 *          ? ASC( cNewbyte )     // result: 23
 *          ? cNewbyte            // result: non-printing character
 *  $END$
 */

FUNCTION FT_BYTEXOR(cByte1, cByte2)

  LOCAL nCounter, xByte

  IF valtype(cByte1) != "C" .or. valtype(cByte2) != "C" // parameter check
     xByte := NIL
  ELSE
     xByte := chr(0)
     FOR nCounter := 0 to 7           // test each bit position
        IF FT_ISBIT(cByte1, nCounter) .or. FT_ISBIT(cByte2, nCounter)
           IF .not. (FT_ISBIT(cByte1, nCounter) .and. FT_ISBIT(cByte2, nCounter))
              xByte := FT_BITSET(xByte, nCounter)
           ENDIF
        ENDIF
     NEXT
  ENDIF

RETURN xByte

