/*
 * File......: CntrySet.Prg
 * Author....: David Husnian
 * Date......: $Date:   02 Apr 1991 18:22:28  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/cntryset.prv  $
 * 
 * This is an original work by David Husnian and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/cntryset.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:22:28   GLENN
*  Nanforum Toolkit
 *
 */


/*  $DOC$
 *  $FUNCNAME$
 *     FT_SETCENTURY()
 *  $CATEGORY$
 *     Environment
 *  $ONELINER$
 *     Check/Set the CENTURY Setting
 *  $SYNTAX$
 *     FT_SETCENTURY( <lNewSetState> ) -> <lOldState>
 *  $ARGUMENTS$
 *     lNewSetState ==> Boolean to Set CENTURY To
 *                      (Logical: Optional, Don't Set)
 *  $RETURNS$
 *     The state of the CENTURY setting upon entry to the routine
 *  $DESCRIPTION$
 *     This function returns the state (ON/OFF, TRUE/FALSE) of the CENTURY
 *     and optionally sets it to a value.
 *  $EXAMPLES$
 *                                      // Get the current CENTURY Setting
 *     lOldState := FT_SETCENTURY()
 *                                      // Get the current CENTURY Setting
 *                                      // and turn it on (set it to TRUE)
 *     lOldState := FT_SETCENTURY(.t.)
 *                                      // Get the current CENTURY Setting
 *                                      // and turn it off (set it to FALSE)
 *     lOldState := FT_SETCENTURY(.f.)
 *  $END$
 */


#define IS_LOGICAL(x)                (VALTYPE(x) == "L")

FUNCTION FT_SETCENTURY(lNewSetState)
                                        // Note that if CENTURY is ON then
                                        // DTOC() Will Return a String of Length
                                        // 10, Otherwise it Will be of Length 8
   LOCAL lOldSetState := (LEN(DTOC(DATE())) == 10)

   IF (IS_LOGICAL(lNewSetState))        // Did They Want it Set??
      SET CENTURY (lNewSetState)        // Yes, Set it
   ENDIF                                // IS_LOGICAL(lNewSetState)
   RETURN (lOldSetState)                // FT_SetCentury
