/*
 * File......: DIR2DBF.PRG
 * Author....: Steve Kolterman
 * CIS ID....: 76320,37
 * Date......: $Date:   02 Apr 1991 18:22:38  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/dir2dbf.prv  $
 * 
 * This is an original work by Steve Kolterman and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/dir2dbf.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:22:38   GLENN
*  Nanforum Toolkit
 *
 */


/*  $DOC$
 *  $FUNCNAME$
 *     FT_DIR2DB()
 *  $CATEGORY$
 *     Environment
 *  $ONELINER$
 *     Fill a DBF with filenames
 *  $SYNTAX$
 *     FT_DIR2DB( <spec>, <dbf>, <ntx> ) -> NIL
 *  $ARGUMENTS$
 *
 *  $RETURNS$
 *     NIL
 *  $DESCRIPTION$
 *     Creates/overwrites and fills a specified .DBF from a DOS file
 *     spec, and optionally, creates an .NTX on the 'name' field.
 *
 *     Because of the certitude of everyone having their own
 *     ways of displaying warning messages, this function does NOT
 *     care if a .DBF or .NTX of the same name as the <dbf> and
 *     <ntx> params. already exists.  Check first, as in the sample
 *     usage above, or modify the source to write those warnings in
 *     your own style.  Also, you are returned to the work area your
 *     program was in prior to the call.
 *  $EXAMPLES$
 *     IF !FILE("allfiles.dbf")
 *       FT_DIR2DB( "\data\*.*","allfiles","namentx" )
 *     ENDIF
 *
 *     Creates ALLFILES.DBF and NAMENTX.NTX and fills with all files
 *      in the '\data' subdir.  To exclude the .NTX creation, don't
 *      pass the <ntx> parameter.
 *
 *  $END$
 */

#ifdef FT_TEST
   FUNCTION MAIN()
	   ? select()
         FT_DIR2DB( "*.*","allfiles","namentx" )
	   ? select()
   return ( nil )
#endif
 
FUNCTION FT_DIR2DB( spec, dfilename, ntxfile )
  LOCAL struc   := DIRECTORY(spec),  ;
        doindex := (ntxfile != NIL), ; 
        o_area  := SELECT(),         ;
        adbf    := { {"Name","C",12,0},{"Size","N",9,0},{"Date","D",8,0},;
				   {"Time","C",8,0},{"Attr","C",1,0} }
  FIELD name

  DBCREATE( dfilename, adbf)
  USE (dfilename) NEW
  Aeval( struc, { |x,y| _FTAddFill( y,@struc,@adbf ) } )
  IF doindex
     INDEX ON name TO (ntxfile)
  ENDIF
  SELECT(o_area)
  RETURN NIL


STATIC FUNCTION _FTAddFill( elem_num,struc,adbf )
  APPEND BLANK
  AEVAL ( adbf, { |x,y| FIELDPUT( y,struc[ elem_num ][ y ] ) } )
  RETURN NIL

