/*
 * File......: HEX2DEC.PRG
 * Author....: Robert A. DiFalco
 * Date......: $Date:   02 Apr 1991 18:23:02  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/hex2dec.prv  $
 * 
 * This is an original work by Robert DiFalco and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/hex2dec.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:23:02   GLENN
*  Nanforum Toolkit
 *
 */

/*
 * $DOC$
 * $FUNCNAME$
 *    FT_HEX2DEC()
 *  $CATEGORY$
 *    Conversion
 * $ONELINER$
 *    Convert a hex number to decimal
 * $SYNTAX$
 *    FT_HEX2DEC( <cHexNum> ) -> nDecNum
 * $ARGUMENTS$
 *    <cHexNum> is a character string representing a hex number.
 * $RETURNS$
 *    A decimal number.
 * $DESCRIPTION$
 *    Converts a hexidecimal number to a BASE 10 decimal number.
 *    Useful for using FT_INT86().
 * $EXAMPLES$
 *    FT_INT86( Hex2Dec( "21" ), aRegs )
 *
 *    Converts 21h, the Dos Interrupt, to its decimal equivelant,
 *    33, for use by FT_INT86().
 * $END$
 */

#define HEXTABLE "0123456789ABCDEF"

#ifdef FT_TEST
  FUNCTION MAIN( cHexNum )
     QOut( FT_HEX2DEC( cHexNum ) )
     return ( nil )
#endif

FUNCTION FT_HEX2DEC( cHexNum )
   local n, nDec := 0, nHexPower := 1

   for n := len( cHexNum ) to 1 step -1
      nDec += ( at( subs( upper(cHexNum), n, 1 ), HEXTABLE ) - 1 ) * nHexPower
      nHexPower *= 16
   next

RETURN nDec

