/*
 * File......: ISBIT.PRG
 * Author....: Forest Belt, Computer Diagnostic Services, Inc.
 * Date......: $Date:   02 Apr 1991 18:23:08  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/isbit.prv  $
 * 
 * This is an original work by Forest Belt and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/isbit.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:23:08   GLENN
*  Nanforum Toolkit
 *
 */


/*  $DOC$
 *  $FUNCNAME$
 *     FT_ISBIT()
 *  $CATEGORY$
 *     String
 *  $ONELINER$
 *     Test the status of an individual bit
 *  $SYNTAX$
 *     FT_ISBIT( <cByte>, <nBitPos> ) -> lResult
 *  $ARGUMENTS$
 *     <cByte> is a character from CHR(0) to CHR(255)
 *
 *     <nBitpos> is a number from 0 to 7 conforming to standard right-to-left
 *               bit-numbering convention and representing the position of the
 *               bit within the byte.
 *  $RETURNS$
 *     .T. if designated bit is set (1), .F. if not set (0), NIL if
 *      invalid parameters.
 *  $DESCRIPTION$
 *     Tests for status of any selected bit in the byte passed as a parameter.
 *     Byte must be presented in CHR() form, as a literal constant, or as the
 *     one-byte-string result of an expression.
 *  $EXAMPLES$
 *     This code tests whether bit 3 is set in the byte represented by
 *     chr(107):
 *
 *      lBitflag := FT_ISBIT(chr(107), 3)
 *      ? lBitflag                  // result: .T.
 *
 *      This code tests whether bit 5 is set in the byte represented by ASCII
 *      65 (letter 'A')
 *
 *      ? FT_ISBIT('A', 5)          // result: .F.
 *  $END$
 */
 
FUNCTION FT_ISBIT(cInbyte,nBitPos)

  LOCAL xBitStat

  IF valtype(cInbyte) != "C" .or. valtype(nBitPos) != "N"  // parameter check
     xBitStat := NIL
  ELSE
     if (nBitPos > 7) .or. (nBitPos < 0) .or. (nBitPos != int(nBitPos))
        xBitStat := NIL
     else
        xBitStat := int(((asc(cInByte) * (2 ^ (7 - nBitPos))) % 256) / 128) == 1
     endif
  ENDIF

RETURN xBitStat

