/*
 * File......: LASTDAY.PRG
 * Author....: Jeff Bryant
 * Date......: $Date:   02 Apr 1991 18:23:20  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/lastday.prv  $
 * 
 * This is an original work by Jeff Bryant and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/lastday.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:23:20   GLENN
*  Nanforum Toolkit
 *
 */


/*  $DOC$
 *  $FUNCNAME$
 *     FT_LDAY()
 *  $CATEGORY$
 *     Date/Time
 *  $ONELINER$
 *     Returns last day of the month
 *  $SYNTAX$
 *     FT_LDAY( [<dDateToChk>] ) -> dLastDay
 *  $ARGUMENTS$
 *     <dDateToChk> is a date within a month for which you want to find
 *     the last date of that month.  If not passed or is an incorrect type,
 *     defaults to current system date.
 *  $RETURNS$
 *     A Clipper date value representing the last date of the month.
 *  $DESCRIPTION$
 *     This function will return the first day of the month of the date
 *     passed, or the first day of the current month if no argument is
 *     supplied.
 *  $EXAMPLES$
 *     dDate := CTOD( "09/15/90" )
 *     ? FT_LDAY( dDate )             // 09/30/90
 *     ? FT_LDAY()                    // 03/31/91  (current month)
 *  $SEEALSO$
 *     FT_FDAY()
 *  $END$
 */

FUNCTION FT_LDAY(dDateToChk)

   LOCAL nCheckMonth, nLastDay

   IF VALTYPE(dDateToChk) # "D"
      dDateToChk := Date()
   ENDIF

   nCheckMonth := MONTH(dDateToChk)

   IF nCheckMonth == 4 .OR. nCheckMonth == 6 .OR. ;
      nCheckMonth == 9 .OR. nCheckMonth == 11
      nLastDay := 30
   ELSEIF nCheckMonth == 2
      nLastDay := IIF(YEAR(dDateToChk) % 4 == 0, 29, 28)
   ELSE
      nLastDay := 31
   ENDIF

   RETURN dDateToChk + (nLastDay - DAY(dDateToChk))

