/*
 * File......: MOUSE1.PRG
 * Author....: Robert DiFalco
 * Date......: $Date:   02 Apr 1991 18:23:34  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/mouse1.prv  $
 * 
 * This is an original work by Robert DiFalco and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/mouse1.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:23:34   GLENN
*  Nanforum Toolkit
 *
 */

#include "FTINT86.CH"

/*
      These statics create a Mouse Event
      record.
 */

static nMouseX, nMouseY
static aPrsCnt[2], aLstPrsX[2], aLstPrsY[2]
static aRelCnt[2], aLstRelX[2], aLstRelY[2]
static nHorMcks, nVerMcks
static aReg[12]
static lCrsState


#ifdef FT_TEST

  FUNCTION MAIN()


     local nX, nY, cSavClr
     local cSavScr := savescreen( 0, 0, maxrow(), maxcol() )

     if empty( FT_MRESET() )
        @ maxrow(), 0 say "Mouse driver is not installed!"
        return ""
     endif

     * ..... Set up the screen
     cSavClr := setcolor( "w/n" )
     @ 0,0,maxrow(),maxcol() box ""

     setcolor( "GR+/RB" )
     scroll( 7,2,19,60,0 )
     @ 7,2 to 19,60

     @ 16, 10 to 18, 40 double

     setcolor( "N/W" )
     @ 17, 11 say "  Double Click here to Quit  "

     setcolor( "GR+/RB" )

     * ..... Start the demo

     FT_MSHOWCRS()

     do while .t.
        nX := nY := 0

        devpos( 9, 10 )
        devout( "FT_MMICKEYS:" )
        FT_MMICKEYS( @nX, @nY )
        devout( nX )
        devout( nY )

        devpos( 10, 10 )
        devout( "FT_MGETPOS :" )
        DEVOUT( FT_MGETPOS( @nX, @nY ) )
        devout( nX )
        devout( nY )

        devpos( 11, 10 )
        devout( "FT_MGETX   :" )
        DEVOUT(  FT_MGETX() )

        devpos( 12, 10 )
        devout( "FT_MGETY   :")
        DEVOUT( FT_MGETY() )

        devpos( 13, 10 )
        devout( "FT_MBUTPRS :" )
        DEVOUT( FT_MBUTPRS(2) )

        devpos( 14, 10 )
        devout( "FT_MBUTREL :" )
        DEVOUT( FT_MBUTREL(2) )

        if FT_MINREGION( 17, 11, 17, 39 )
           if FT_MDBLCLK()
              exit
           endif
        endif

     enddo

     FT_MHIDECRS()

     setcolor( cSavClr )
     restscreen( 0, 0, maxrow(), maxcol(), cSavScr )
     devpos( maxrow(), 0 )
  RETURN nil


#endif 





/*
 * $DOC$         
 * $FUNCNAME$
 *    FT_MRESET()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Reset mouse driver and return status of mouse
 * $SYNTAX$
 *    FT_MRESET() -> nMouseStatus
 * $ARGUMENTS$
 *    NONE
 * $RETURNS$
 *    An integer representing the mouse status (0 == mouse not installed)
 * $DESCRIPTION$
 *    Resets the mouse drive and returns mouse status. Use FT_MSHOWCRS()
 *    to display the mouse cursor.
 * $EXAMPLES$
 *    IF Empty( FT_MRESET() )
 *       ? "No mouse driver is installed"
 *    ENDIF
 * $END$
 */

FUNCTION FT_MRESET()

   aReg[1] := 0          // set mouse function call 0
   FT_INT86( 51, aReg )  // execute mouse interrupt

RETURN aReg[2] 	       // return status code


/* $DOC$         
 * $FUNCNAME$
 *    FT_MCURSOR()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Set the mouse cursor
 * $SYNTAX$
 *    FT_MCURSOR( <lStat> ) -> lCursorState
 * $ARGUMENTS$ 
 *    <lState> is a logical indicating whether to set the mouse cursor on.
 * $RETURNS$
 *    A locical indicating the previous mouse cursor state.
 * $DESCRIPTION$
 *    This function works like most Clipper state functions.  If no value
 *    is sent to FT_MCURSOR() it will return the state of the mouse cursor.
 * $EXAMPLES$
 *    IF !( FT_MCURSOR() )
 *       FT_MCURSOR( .T. )
 *    ENDIF
 *
 * $END$
 */

FUNCTION FT_MCURSOR( lState )
   local lSavState := lCrsState

   lState := if( lState == NIL, lCrsState, lState )

   if ( lCrsState := lState )
      FT_MSHOWCRS()
   else
      FT_MHIDECRS()
   endif

RETURN lSavState


/* $DOC$         
 * $FUNCNAME$
 *    FT_MSHOWCRS()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Increment internal cursor flag and display cursor
 * $SYNTAX$
 *    FT_MSHOWCRS() -> NIL
 * $ARGUMENTS$
 *    NONE
 * $RETURNS$
 *    NIL
 * $DESCRIPTION$
 *    Displays the mouse cursor. Make sure to turn the mouse cursor off
 *    when redrawing screens.  The mouse cursor dutifully saves the screen
 *    under it, so if you draw over the mouse cursor it will create a "hole"
 *    in your screen when you move the mouse cursor.
 * $EXAMPLES$
 *    IF Empty( FT_MRESET() )
 *       FT_MSHOWCRS()
 *    ENDIF
 * $END$
 */

FUNCTION FT_MSHOWCRS()

   aReg[1] := 1         // set mouse function call 1
   FT_INT86( 51, aReg ) // execute mouse interrupt
   lCrsState := .t.

RETURN NIL              // no output from function



/*
 * $DOC$         
 * $FUNCNAME$
 *    FT_MHIDECRS()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Decrements internal mouse cursor flag and hides mouse cursor
 * $SYNTAX$
 *    FT_MHIDECRS() -> NIL
 * $ARGUMENTS$
 *    NONE
 * $RETURNS$
 *    NIL
 * $DESCRIPTION$
 *    Hides the mouse cursor. Make sure to turn the mouse cursor off when
 *    redrawing screens.  The mouse cursor dutifully saves the screen
 *    under it, so if you draw over the mouse cursor it will create a
 *    "hole" in your screen when you move the mouse cursor.
 *
 * $EXAMPLES$
 *    FT_MHIDECRS()
 *    @ 10, 10 to 20, 20
 *    FT_MSHOWCRS()
 *
 * $END$
 */



FUNCTION FT_MHIDECRS()   // decrement internal cursor flag and hide cursor

   aReg[1] := 2          // set mouse function call 2
   FT_INT86( 51, aReg )  // execute mouse interrupt
   lCrsState := .f.

RETURN NIL		          // no output from function


/*
 * $DOC$         
 * $FUNCNAME$
 *    FT_MGETPOS()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Get mouse cursor position and button status
 * $SYNTAX$
 *    FT_MGETPOS( @<nX>, @<nY> ) -> nButtonStatus
 * $ARGUMENTS$ 
 *    <nX> is a variable that will receive the mouse X position.  It must
 *    be passed by reference.
 *
 *    <nY> is a variable that will receive the mouse Y position.  It must
 *    be passed by reference.
 * $RETURNS$
 *    Button Status
 * $DESCRIPTION$
 *    Loads cursor position into x and y coordinates passed by reference and
 *    returns the button status.
 * $EXAMPLES$
 *    LOCAL nX, nY
 *    LOCAL nButton := FT_MGETPOS( @nX, @nY )
 *    ? "Mouse Row    :", nX
 *    ? "Mouse Column :", nY
 *    ? "Button Status:", nButton
 *
 * $END$
 */




FUNCTION FT_MGETPOS( nX, nY )

   nX := if( nX == NIL, 0, nX )
   nY := if( nY == NIL, 0, nX )

   aReg[1] := 3                // set mouse function call 3
   FT_INT86( 51, aReg )        // execute mouse interrupt
   nX := nMouseX := aReg[4]    // store new x-coordinate
   nY := nMouseY := aReg[3]    // store new y-coordinate

RETURN aReg[2]                 // return button status

/* $DOC$         
 * $FUNCNAME$
 *    FT_MGETX()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Get mouse cursor row position
 * $SYNTAX$
 *    FT_MGETX() -> nRowPos
 * $ARGUMENTS$
 *    NONE
 * $RETURNS$
 *    Row position of mouse
 * $DESCRIPTION$
 *    Retrieves mouses row position
 * $EXAMPLES$ 
 *    ? FT_MGETX()
 * $END$
 */

FUNCTION FT_MGETX()
   FT_MGETPOS()
RETURN int( nMouseX / 8 )

/*
 * $DOC$         
 * $FUNCNAME$
 *    FT_MGETY()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Get mouse cursor column position
 * $SYNTAX$
 *    FT_MGETY() -> nColPos
 * $ARGUMENTS$
 *    NONE
 * $RETURNS$
 *    Column position of mouse.
 * $DESCRIPTION$
 *    Retrieves mouses column position.
 * $EXAMPLES$
 *    ? FT_MGETY()
 * $END$
 */

FUNCTION FT_MGETY()
   FT_MGETPOS()
RETURN int( nMouseY / 8 )

/*
 * $DOC$         
 * $FUNCNAME$
 *    FT_MSETPOS()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Position the mouse cursor
 * $SYNTAX$
 *    FT_MSETPOS( <nX>, <nY> ) -> NIL
 * $ARGUMENTS$ 
 *    <nX> is the desired mouse row.
 *
 *    <nY> is the desired mouse column.
 * $RETURNS$
 *    NIL
 * $DESCRIPTION$
 *    Positions mouse cursor on screen.
 * $EXAMPLES$
 *    FT_MSETPOS( 10, 20 )     // position mouse cursor at row 10, col 20
 * $END$
 */

FUNCTION FT_MSETPOS( nX, nY )  // set mouse cursor location

   aReg[1] := 4                // set mouse function call 4
   aReg[3] := nX               // assign new x-coordinate
   aReg[4] := nY               // assign new y-coordinate
   FT_INT86( 51, aReg )        // execute mouse interrupt

RETURN NIL                     // no function output

/*
 * $DOC$         
 * $FUNCNAME$
 *    FT_MXLIMIT()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Set vertical bounds of mouse
 * $SYNTAX$
 *    FT_MXLIMIT( <nX1>, <nX2> ) -> NIL
 * $ARGUMENTS$
 *    <nX1> is the top row limit.
 *
 *    <nX2> is the bottom row limit.
 * $RETURNS$
 *    NIL
 * $DESCRIPTION$
 *    Set maximum vertical bounds of mouse.
 * $EXAMPLES$
 *    FT_MXLIMIT( 10, 20 )
 * $END$
 */

FUNCTION FT_MXLIMIT( nXMin, nXMax )   // set vertical minimum and maximum coordinates

   aReg[1] = 7                        // set mouse function call 7
   aReg[3] = nXMin                    // load vertical minimum parameter
   aReg[4] = nXMax                    // load vertical maximum parameter
   FT_INT86( 51, aReg )               // execute mouse interrupt

RETURN NIL

/*
 * $DOC$         
 * $FUNCNAME$
 *    FT_MYLIMIT()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Set horizontal bounds of mouse
 * $SYNTAX$
 *    FT_MYLIMIT( <nY1>, <nY2> ) -> NIL
 * $ARGUMENTS$
 *    <nY1> is the left column limit.
 *
 *    <nY2> is the right column limit.
 * $RETURNS$
 *    NIL
 * $DESCRIPTION$
 *    Set maximum horizontal bounds of mouse.
 * $EXAMPLES$
 *    FT_MYLIMIT( 10, 20 )
 * $END$
 */

FUNCTION FT_MYLIMIT( nYMin, nYMax )  // set horizontal minimum and maximum coordinates

   aReg[1] = 8                       // set mouse function call 8
   aReg[3] = nYMin                   // load horz minimum parameter
   aReg[4] = nYMax                   // load horz maximum parameter
   FT_INT86( 51, aReg )              // execute mouse interrupt

RETURN NIL                           // no function output


/*
 * $DOC$         
 * $FUNCNAME$
 *    FT_MBUTPRS()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Retrieve button press status
 * $SYNTAX$
 *    FT_MRESET( <nButton> ) -> nButStatus
 * $ARGUMENTS$
 *    <nButton> is the mouse button number:
 *
 *               0 -- Left   Button
 *               1 -- Right  Button
 *               2 -- Middle Button [if applicable]
 * $RETURNS$
 *    An integer representing the button status:
 *
 *               0 - no buttons pressed
 *               1 - left button pressed
 *               2 - right button pressed
 *               3 - left and right pressed
 *               4 - middle pressed
 *               5 - left and middle pressed
 *               6 - middle and right buttons pressed
 *               7 all 3 buttons pressed
 *
 * $DESCRIPTION$
 *    Retrieves the mouse button status.
 * $EXAMPLES$
 *    IF Empty( FT_MBUTPRS(1) )
 *       ? "No Item selected"
 *    ENDIF
 *
 * $END$
 */

FUNCTION FT_MBUTPRS( nButton ) // get button press information

   aReg[1] := 5	             // set mouse function call 5
   aReg[2] := nButton          // pass parameter for left or right button
   FT_INT86( 51, aReg )        // execute mouse interrupt
   aPrsCnt[nButton]  = aReg[2] // store updated press count
   aLstPrsX[nButton] = aReg[3] // x-coordinate at last press
   aLstPrsY[nButton] = aReg[4] // y-coordinate at last press

RETURN aReg[1]                 // return button status



/*  $DOC$
 *  $FUNCNAME$
 *     FT_MBUTREL()
 *  $CATEGORY$
 *     Keyboard/Mouse
 *  $ONELINER$
 *     Get mouse button release information
 *  $SYNTAX$
 *     FT_MBUTREL( nButton ) -> nReleaseInfo
 *  $ARGUMENTS$
 *     <nButton> is the mouse button number
 *  $RETURNS$
 *     <nBStat>  = Button Status
 *                 0 = NONE
 *                 1 = LEFT
 *                 2 = RIGHT
 *                 3 = MIDDLE
 *
 *  $DESCRIPTION$
 *     This function returns the release status of the mouse buttons.
 *  $EXAMPLES$
 *     IF( FT_MBUTREL( 0 ) == 1 )
 *        ? "Left button released"
 *     ENDIF
 *
 *  $END$
 */



FUNCTION FT_MBUTREL( nButton ) // get button release information

   aReg[1] := 6                // set mouse function call 6
   aReg[2] := nButton          // pass parameter for left or right button
   FT_INT86( 51, aReg )        // execute mouse interrupt
   aRelCnt[nButton] = aReg[2]  // store updated release count 
   aLstRelX[nButton] = aReg[3] // x-coordinate at last release
   aLstRelY[nButton] = aReg[4] // y-coordinate at last release

RETURN aReg[1]                 // return button status


/*
 * $DOC$         
 * $FUNCNAME$
 *    FT_MDEFCRS()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Defines the mouse cursor
 * $SYNTAX$
 *    FT_MDEFCRS( <nCrsType>, <nScrMask>, <nCrsMask> ) -> NIL
 * $ARGUMENTS$
 *    <nCrsType> is the cursor type.
 *
 *    <nScrMask> is the screen mask.
 *
 *    <nCrsMask> is the cursor mask.
 * $RETURNS$
 *    NIL
 * $DESCRIPTION$
 *    Defines the cursor type
 * $END$
 */

FUNCTION FT_MDEFCRS( nCurType, nScrMask, nCurMask )   // define text cursor type and masks

   aReg[1] = 10   	   // set mouse function call 10
   aReg[2] = nCurType   // load cursor type parameter
   aReg[3] = nScrMask   // load screen mask value
   aReg[4] = nCurMask   // load cursor mask value
   FT_INT86( 51, aReg ) // execute mouse interrupt

RETURN NIL              // no function output


/*
 * $DOC$         
 * $FUNCNAME$
 *    FT_MMICKEYS()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Get mickeys
 * $SYNTAX$
 *    FT_MMICKEYS( @<nX>, @<nY> ) -> NIL
 * $ARGUMENTS$
 *    <nX> is a variable that will recieve the vertical mickey count.
 *
 *    <nY> is a variable that will recieve the horizontal mickeys count.
 * $RETURNS$
 *    NIL
 * $DESCRIPTION$
 *    <nX> and <nY> must be passed by reference to recieve
 *    the mouse position in Mickeys.
 * $EXAMPLES$
 *    FT_MMICKEYS( @nX, @nY )
 *    ? nX
 *    ? nY
 * $END$
 */

FUNCTION FT_MMICKEYS( nX, nY ) // read mouse motion counters

   aReg[1] = 11                // set mouse function call 11
   FT_INT86( 51, aReg )        // execute mouse interrupt
   nX := nHorMcks := aReg[3]   // store horizontal motion units
   nY := nVerMcks := aReg[4]   // store vertical motion units

RETURN NIL                     // no function output


/*
 * $DOC$         
 * $FUNCNAME$
 *    FT_MDBLCLK()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Returns true if a double click was detected
 * $SYNTAX$
 *    FT_MDBFCLK( <nClick> ) -> lIsDoubleClk
 * $ARGUMENTS$
 *    <nClick> is a numeric value.  If it is a non - zero value. FT_MDBLCLK()
 *    will not check the first press but rather will check for two releases.
 * $RETURNS$
 *    .T. if a double click was detected.
 * $DESCRIPTION$
 *    This is a mouse meta function that checks for the presence
 *    of a double click.
 * $EXAMPLES$
 *   IF FT_MISREGION( 10, 10, 11, 20 ) .AND. FT_MDBLCLK()
 *      MnuItem1()
 *   ENDIF
 *
 * $END$
 */

FUNCTION FT_MDBLCLK( nClick )

   local nInterval := 0.5
   local nVert  := FT_MGETX()
   local nHorz  := FT_MGETY()
   local nStart := seconds()

   nClick := if( nClick == NIL, 0, nClick )

   // Wait for first press
   do while empty( nClick ) .and. ;
            seconds() - nStart < nInterval .and. ;
            FT_MGETX() == nVert .and. ;
            FT_MGETY() == nHorz

      nClick := FT_MBUTPRS( 1 )

   enddo

   // Wait for first release
   do while !empty( nClick ) .and. ;
            seconds() - nStart < nInterval .and. ;
            FT_MGETX() == nVert .and. ;
            FT_MGETY() == nHorz

      nClick := FT_MBUTPRS( 1 )

   enddo

   // Wait for second click
   do while empty( nClick ) .and. ;
            seconds() - nStart < nInterval .and. ;
            FT_MGETX() == nVert .and. ;
            FT_MGETY() == nHorz

      nClick := FT_MBUTPRS( 1 )

   enddo

   // Wait for second release
   do while !empty( nClick ) .and. ;
            seconds() - nStart < nInterval .and. ;
            FT_MGETX() == nVert .and. ;
            FT_MGETY() == nHorz

      nClick := FT_MBUTPRS( 1 )

   enddo

RETURN ( seconds() - nStart < nInterval .and. ;
         FT_MGETX() == nVert .and. ;
         FT_MGETY() == nHorz )

/*
 * $DOC$         
 * $FUNCNAME$
 *    FT_MINREGION()
 * $CATEGORY$
 *    Keyboard/Mouse
 * $ONELINER$
 *    Test if the mouse cursor is in the passed region
 * $SYNTAX$
 *    FT_MINREGION( <nT>, <nL>, <nB>, <nR> ) ---> lInRegion
 * $ARGUMENTS$
 *    <nT>, <nL> <nB> <nR> are the four corners of the screen region.
 * $RETURNS$ 
 *    .T. if mouse is in specified region.
 * $DESCRIPTION$
 *    This function will check to see if the mouse cursor is
 *    within the limits of the specified region.
 * $EXAMPLES$
 *    IF FT_MINREGION( 10, 10, 11, 20 )
 *      nChoice := 1
 *    ENDIF
 *
 * $END$
 */

FUNCTION FT_MINREGION( nTR, nLC, nBR, nRC )
RETURN ( FT_MGETX() >= nTR .and. FT_MGETX() <= nBR .and. ;
         FT_MGETY() >= nLC .and. FT_MGETY() <= nRC )

