/*
 * File......: NWLSTAT.PRG
 * Author....: Glenn Scott
 * Date......: $Date:   02 Apr 1991 18:23:40  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/nwlstat.prv  $
 * 
 * This is an original work by Glenn Scott and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/nwlstat.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:23:40   GLENN
*  Nanforum Toolkit
 *
 */


/*  $DOC$
 *  $FUNCNAME$
 *     FT_NWLSTAT()
 *  $CATEGORY$
 *     Netware
 *  $ONELINER$
 *     Return the current Novell Netware logical station number
 *  $SYNTAX$
 *     FT_NWLSTAT() -> nStatNum
 *  $ARGUMENTS$
 *     None
 *  $RETURNS$
 *     A numeric corresponding to the current logical station number
 *     assigned by Netware. 
 *  $DESCRIPTION$
 *     In order to find out information about a particular node logged
 *     in to a Netware server, you will need the logical 
 *     station number, also known as a "connection number."  This 
 *     function will return that number.  This will be a number from 1 to 100
 *     under Netware 286, or 1-250 under Netware 386.  This is *not* the same
 *     as a physical station number.
 *
 *     This function requires FT_INT86().
 *
 *     This function does NOT test for the existence of the Netware shell.  
 *     The behavior is undefined if no shell is loaded. 
 *  $EXAMPLES$
 *     QOut( "Logical station: " + str( FT_NWLSTAT() ) )
 *  $END$
 */

#include "FTINT86.CH"

#define DOS         33
#define STATNUM    220
#define LOBYTE( x )    ( Int( iif( x < 0, x + 256, x ) % 256) )

#ifdef FT_TEST
  FUNCTION MAIN()
  QOut( "Logical station: " + str( FT_NWLSTAT() ) )
  return ( nil )
#endif

FUNCTION FT_NWLSTAT()
  LOCAL aRegs[ INT86_MAX_REGS ]
  LOCAL nStation

  aRegs[ AX ] = MAKEHI( STATNUM )
  FT_INT86( DOS, aRegs )

  nStation := LOWBYTE( aRegs[ AX ] )
  if nStation < 0
    nStation += 256
  endif

  RETURN nStation
