/*
 * File......: SCANCODE.PRG
 * Author....: Glenn Scott (from John Kaster)
 * Date......: $Date:   02 Apr 1991 18:24:00  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/scancode.prv  $
 * 
 * This is an original work by Glenn Scott and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/scancode.prv  $
*  
*     Rev 1.0   02 Apr 1991 18:24:00   GLENN
*  Nanforum Toolkit
 *
 */


/*  $DOC$
 *  $FUNCNAME$
 *     FT_SCANCODE()
 *  $CATEGORY$
 *     Keyboard/Mouse
 *  $ONELINER$
 *     Wait for keypress and return keyboard scan code
 *  $SYNTAX$
 *     FT_SCANCODE() -> cCode
 *  $ARGUMENTS$
 *     None
 *  $RETURNS$
 *     A two-character string, corresponding to the keyboard scan code.
 *  $DESCRIPTION$
 *
 *     FT_SCANCODE() enables you to distinguish the different scancodes
 *     of similar keys (such as Grey minus versus regular minus), thus
 *     increasing the number of keys your input routine can recognize.
 *
 *     It works like INKEY(), in that it waits for a key to be pressed.
 *     The scan code consists of two bytes, which are returned as a
 *     two-character string.
 *
 *     For example, calling FT_SCANCODE() and pressing the Grey-minus
 *     key will return a two character string:
 *
 *            chr(45) + chr(74)
 *
 *     LASTKEY() is not updated by FT_SCANCODE(), so don't try to
 *     test LASTKEY() to see what was pressed during an FT_SCANCODE()
 *     call.  Simply assign the return value to a variable and test
 *     that (see the test driver below).
 *
 *     *  This was adapted from a short C routine posted by John Kaster on
 *        NANFORUM.  It was written in Clipper to help demonstrate the 
 *        FT_INT86 FUNCTION OF THE NANFORUM TOOLKIT.
 *
 *     THIS PROGRAM REQUIRES FT_INT86().
 *
 *  $EXAMPLES$
 *
 *        cKey := FT_SCANCODE()
 * 
 *
 *      [grey-] returns:  chr(45)+chr(74)
 *      [-]     returns:  chr(45)+chr(12)
 *      [grey+] returns:  chr(43)+chr(78)
 *      [+]     returns:  chr(43)+chr(13)
 *
 *  $END$
 */

#include "FTINT86.CH"

#define KEYB       22

#ifdef FT_TEST

  #DEFINE SCANCODE_ESCAPE   (chr(27) + chr(1))

  FUNCTION main()
     LOCAL getlist, cKey
     CLEAR
     QOut("Press any key, ESCape to exit:")

     while .t.
        cKey := FT_SCANCODE()
        QOUT( "chr(" + str(asc(substr(cKey,1,1)),3) + ")+chr(" + str(asc(substr(cKey,2,1)),3) + ")" )
        if cKey == SCANCODE_ESCAPE
           exit
        endif
     end
  RETURN nil
     
#endif

FUNCTION FT_SCANCODE()
  LOCAL aRegs[ INT86_MAX_REGS ]

  aRegs[ AX ] = MAKEHI( 0 )
  FT_INT86( KEYB, aRegs )
  RETURN ( chr(LOWBYTE( aRegs[AX] )) + chr(HIGHBYTE( aRegs[AX] )) )
