/*
 * File......: VIDEO1.C
 * Author....: Robert A. DiFalco
 * Date......: $Date:   02 Apr 1991 18:26:16  $
 * Revision..: $Revision:   1.0  $
 * Log file..: $Logfile:   E:/nanfor/src/video1.c_v  $
 * 
 * This function is an original work by Robert A. DiFalco and is placed
 * in the public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/video1.c_v  $
 * 
 *    Rev 1.0   02 Apr 1991 18:26:16   GLENN
 * Nanforum Toolkit
 *
 */


#include <extend.h>

#define VIDMODE  *(unsigned char far *) 0x00449lu
#define MONOSEG  0xb0000000L
#define COLOR    0xb8000000L
#define VIDSEG   (unsigned char far *)((7 == VIDMODE) ? MONOSEG : COLOR )

unsigned char far *_ftScrPtr( int x, int y );

/*
 *    INTERNALS WARNING! INTERNALS WARNING!
 *    _gtMaxRow() and _gtMaxCol() are internals.  Use with caution.
 *
 */

int _gtMaxRow(void);
int _gtMaxCol(void);


/*  $DOC$
 *  $FUNCNAME$
 *     FT_VIDSTR()
 *  $CATEGORY$
 *     Video
 *  $ONELINER$
 *     Display string on screen in specified attribute
 *  $SYNTAX$
 *     FT_VIDSTR( <nRow>, <nCol>, <cString> [, <nColor>] ) -> NIL
 *  $ARGUMENTS$
 *     <nRow> and <nCol> are the screen coordinates.
 *
 *     <cString> is the string to be printed on the screen.
 *
 *     <nColor> is an integer representing the color attribute.
 *     The formula is:
 *
 *       nFore + ( nBack * 16 )
 *
 *     FT_VIDSTR() will display the string in the current color if
 *     <nColor> is not passed.
 *  $RETURNS$
 *     NIL
 *  $DESCRIPTION$
 *     This is a high speed function to display a string of any ASCII
 *     characters on screen in a specified color attribute.  This function
 *     is useful for constructing screens with a lot of text or repetitive
 *     screen prints where speed is important.
 *  $EXAMPLES$
 *     FT_VIDSTR( 10, 20, "Enter Name :", 165 )
 *             
 *     This example will print the specified text at coordinates 10, 20
 *     in bright white on top of Magenta.
 *
 *  $END$
 */

 
CLIPPER FT_VIDSTR(void)
{
   register unsigned char far *scptr;
   register int    i, j;
   int iTRow   = _parni(1);
   int iLCol   = _parni(2);
   char *cStr  = _parc(3);
   int iAttr   = _parni(4);
   int iLen    = _parclen(3);
   int iIsAttr = _parinfo(4);

   iTRow++;
   iLCol++;
   
   scptr = _xalloc( iLen * 2 );
   for ( i = 0, j = iLCol; i < iLen; ++i, ++j ) {
      scptr =  _ftScrPtr( j, iTRow );
      *scptr = cStr[i];
      scptr++;
      if ( iIsAttr != 0 ) *scptr = (unsigned char) iAttr;
   }
   _xfree( scptr );
   _ret();
}


/*  $DOC$
 *  $FUNCNAME$
 *     FT_WRTCHR()
 *  $CATEGORY$
 *     Video
 *  $ONELINER$
 *     Display character on screen
 *  $SYNTAX$
 *     FT_WRTCHR( <nRow>, <nCol>, <cChar>, <nColor> ) -> NIL
 *  $ARGUMENTS$
 *     <nRow> and <nCol> are the screen coordinates.
 *
 *     <cChar> is the single character to print on the screen.
 *
 *     <nColor> is an integer representing the color attribute.
 *     The formula is:
 *
 *        nFore + ( nBack * 16 )
 *  $RETURNS$
 *     NIL
 *  $DESCRIPTION$
 *     This is a high speed function to display a single ASCII character
 *     on screen in a specified color attribute.  This function is useful
 *     for constructing screens with a lot of text or repetitive screen prints
 *     where speed is important.  It is faster and requires less memory than
 *     FT_VIDSTR().  A working example is contained in ClrTable.Prg.
 *  $EXAMPLES$
 *      FOR nX = 1 to MaxRow()
 *         FOR nY = 1 to MaxCol()
 *            FT_PRNTCHR( nX, nY, "", (nX - 1)+(nY * 16) )
 *         NEXT
 *      NEXT
 *
 *      This example will write the ASCII character 249 to every location
 *      on screen in every possible color combination.  It will recognize
 *      the status of SetBlink().  It uses direct video writes for speed.
 *
 *  $END$
 */

CLIPPER FT_WRTCHR(void)
{
   register unsigned char far *scptr;
   int  iTRow  = _parni(1);
   int  iLCol  = _parni(2);
   char *cChar = _parc(3);
   int  iAttr  = _parni(4);

   iTRow++;
   iLCol++;

   scptr = _xalloc( 4 );
   scptr =  _ftScrPtr( iLCol, iTRow );
   *scptr = cChar[0];             
   scptr++;
   *scptr = (unsigned char) iAttr;
   _xfree( scptr );
   _ret();
}


/*  $DOC$
 *  $FUNCNAME$
 *     FT_CLS()
 *  $CATEGORY$
 *     Video
 *  $ONELINER$
 *     Clear screen
 *  $SYNTAX$
 *     FT_CLS( <nTRow>, <nLCol>, <nBRow>, <nRCol>, <nColor> ) -> NIL
 *  $ARGUMENTS$
 *     <nTRow>, <nLCol>, <nBRow> and  <nRCol> are the screen coordinates
 *     to clear.
 *
 *     <nColor> is an integer representing the color attribute.
 *     The formula is:
 *
 *       nFore + ( nBack * 16 )
 *
 *     The default is black.
 *  $RETURNS$
 *     NIL
 *  $DESCRIPTION$
 *     This is a high speed function to clear the screen at the given
 *     coordinates with the given color attribute.  This does not change
 *     Clipper's color settings.  It uses direct video writes for speed.
 *
 *  $EXAMPLES$
 *     FT_CLS( 0, 0, MaxRow(), MaxCol(), 165 )
 * 
 *     This example will clear the entire screen with the colors
 *     bright white on magenta.
 *  $END$
 */

CLIPPER FT_CLS(void)
{
   register unsigned char far *scptr;
   register int    i, j;
   int iTRow = _parni(1);
   int iLCol = _parni(2);
   int iBRow = _parni(3);
   int iRCol = _parni(4);
   int iAttr = _parni(5);

   int iTR   = _parinfo(1);
   int iLC   = _parinfo(2);
   int iBR   = _parinfo(3);
   int iRC   = _parinfo(4);

   iTRow++;
   iLCol++;

   if( iBR == 0 ) iBRow = _gtMaxRow()+1;
   else iBRow++;
   
   if( iRC == 0 ) iRCol = _gtMaxCol()+1;
   else iRCol++;
  
   for (i = iTRow; i <= iBRow; ++i) {
      scptr =  _ftScrPtr( iLCol, i );
      for (j = iLCol; j <= iRCol; ++j) {
           *scptr = 32;
           scptr++;
           *scptr = (unsigned char) iAttr;
           scptr++;
       }
   }
   _ret();
}



/*  $DOC$
 *  $FUNCNAME$
 *     FT_SETATTR()
 *  $CATEGORY$
 *     Video
 *  $ONELINER$
 *     Change color attributes of screen region
 *  $SYNTAX$
 *     FT_SETATTR( <nTRow>, <nLCol>, <nBRow>, <nRCol>, <nColor> ) -> NIL
 *  $ARGUMENTS$
 *     <nTRow>, <nLCol>, <nBRow>, and <nRCol> are the coordinates of the
 *     screen region.
 *
 *     <nColor> is an integer representing the new color attribute.
 *     The formula is:
 *
 *         nFore + ( nBack * 16 )
 *  $RETURNS$
 *     NIL
 *  $DESCRIPTION$
 *     This is a high speed function to change the colors of a specified
 *     region of the screen without disturbing any text.  Uses direct
 *     video writes.
 *  $EXAMPLES$
 *     FT_SETATTR( 0, 0, MaxRow(), MaxCol(), 165 )
 * 
 *     This example will change the entire screen's colors to bright white on
 *     magenta without changing or overwriting any text on the screen.
 *  $END$
 */

CLIPPER FT_SETATTR(void)
{
   register unsigned char far *scptr;
   register int    i, j;
   int iTRow = _parni(1);
   int iLCol = _parni(2);
   int iBRow = _parni(3);
   int iRCol = _parni(4);
   int iAttr = _parni(5);

   int iTR   = _parinfo(1);
   int iLC   = _parinfo(2);
   int iBR   = _parinfo(3);
   int iRC   = _parinfo(4);

   iTRow++;
   iLCol++;

   if( iBR == 0 ) iBRow = _gtMaxRow()+1;
   else iBRow++;
   
   if( iRC == 0 ) iRCol = _gtMaxCol()+1;
   else iRCol++;
  
   for (i = iTRow; i <= iBRow; ++i) {
       scptr =  _ftScrPtr( iLCol, i );
       scptr++;
       for (j = iLCol; j <= iRCol; ++j) {
           *scptr = (unsigned char) iAttr;
           scptr+= 2;
       }
   }
   _ret();
}



/*  $DOC$
 *  $FUNCNAME$
 *     FT_REVATTR()
 *  $CATEGORY$
 *     Video
 *  $ONELINER$
 *     Reverse colors of specified screen coordinates
 *  $SYNTAX$
 *     FT_REVATTR( <nTRow>, <nLCol>, <nBRow>, <nRCol> ) -> NIL
 *  $ARGUMENTS$
 *     <nTRow>, <nLCol>, <nBRow>, and <nRCol> are the coordinates of the
 *     screen region.
 *  $RETURNS$
 *     NIL
 *  $DESCRIPTION$
 *     This is a high speed function to reverse the color of a specified
 *     screen region without disturbing any text on the screen.  This
 *     function will correctly reverse the color attributes in a region
 *     containing multiple color combinations.
 *  $EXAMPLES$
 *     FT_REVATTR( 0, 0, MaxRow(), MaxCol() )
 *  
 *     This example will change the entire screen's colors to their reverse
 *     attributes without changing  or overwriting any text.
 *  $END$
 */

CLIPPER FT_REVATTR(void)
{
   register unsigned char far *scptr;
   register int i, j, iFore, iBack;
   int iTRow = _parni(1);
   int iLCol = _parni(2);
   int iBRow = _parni(3);
   int iRCol = _parni(4);

   iTRow++;
   iLCol++;
   iBRow++;
   iRCol++;
   for (i = iTRow; i <= iBRow; ++i) {
       scptr =  _ftScrPtr( iLCol, i );
       scptr++;
       for (j = iLCol; j <= iRCol; ++j) {
          iFore = *scptr % 16;
          iBack = (*scptr - iFore)/16;
          *scptr = (unsigned char) ((iFore * 16) + iBack);
          scptr+= 2;
       }
   }
   _ret();
}



/*  $DOC$
 *  $FUNCNAME$
 *     FT_REVCHR()
 *  $CATEGORY$
 *     Video
 *  $ONELINER$
 *     Reverse the color of a single character on the screen
 *  $SYNTAX$
 *     FT_REVCHR( <nTRow>, <nLCol> ) -> NIL
 *  $ARGUMENTS$
 *     <nTRow>, <nLCol> are the screen coordinates of the character.
 *  $RETURNS$
 *     NIL
 *  $DESCRIPTION$
 *     This is a high speed function to reverse the color of a single
 *     character on the screen without changing the character itself.
 *     This function is the same as FT_REVATTR() except that it changes
 *     only one character on screen and hence is faster and uses less memory.
 *  $EXAMPLES$
 *     FT_REVCHR( 10, 20 )
 * 
 *     This example will change the text and background at 10, 20 to it's
 *     reverse color attributes without changing or overwriting the
 *     character itself.
 *  $END$
 */

CLIPPER FT_REVCHR(void)
{
   register unsigned char far *scptr;
   int iFore, iBack;
   
   int iTRow = _parni(1);
   int iLCol = _parni(2);

   iTRow++;
   iLCol++;

   scptr = _xalloc( 4 );
   scptr =  _ftScrPtr( iLCol, iTRow );
   scptr++;
   iFore = *scptr % 16;
   iBack = (*scptr - iFore)/16;
   *scptr = (unsigned char) ((iFore * 16) + iBack);
   _xfree( scptr );
   _ret();
}


unsigned char far *_ftScrPtr( int iCol, int iRow )
{
	return( VIDSEG + (--iRow * 160) + (--iCol << 1) );
}
